
# q-expansion of newform 8673.2.a.j, downloaded from the LMFDB on 29 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8673
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, -1], [-1, 0], [3, 0], [0, 0], [-3, 4], [-3, 6], [1, 1], [1, 3], [-3, 1], [-6, 1], [4, -1], [-3, 5], [-2, 5], [3, 6], [0, 3], [-1, 4], [-1, 0], [-4, -5], [-1, -2], [1, 2], [0, -5], [-3, 0], [6, -3], [2, -5], [3, 4], [10, -2], [16, 2], [11, -13], [5, -3], [9, -2], [-9, 4], [-10, 2], [-11, 4], [9, 6], [-1, -13], [4, 3], [5, -8], [-8, -1], [16, -7], [-16, 1], [3, -10], [-6, 9], [-9, -6], [-20, 12], [-6, 10], [-14, 17], [3, 1], [-4, 12], [11, -1], [6, -21], [2, 2], [-15, 6], [0, 4], [-7, 8], [12, -16], [-2, -1], [-21, 10], [15, -10], [5, 0], [12, -18], [-7, -3], [9, -19], [12, 0], [3, -21], [13, 9], [3, 0], [18, -2], [-21, -9], [16, -19], [-23, 0], [-9, 15], [-4, -16], [15, -12], [-1, -9], [-13, 12], [-25, -8], [14, -16], [27, -12], [-21, 9], [-6, -12], [-2, -8], [3, -12], [4, -17], [-15, 3], [-3, -3], [-15, 9], [1, -26], [6, -3], [-7, -7], [3, -13], [-12, -8], [13, -17], [-3, 9], [-22, 29], [5, 0], [28, 5], [6, 23], [-2, -23], [29, -20], [-7, 6], [11, -21], [-14, -8], [31, -4], [-26, 23], [19, -24], [9, -20], [-27, 26], [-22, -7], [8, -10], [25, -13], [39, -14], [36, -6], [9, -35], [17, -26], [15, -12], [17, -22], [-9, -1], [-8, 4], [22, 7], [1, 5], [-24, 9], [-10, 0], [12, 10], [13, 10], [-22, -14], [-26, 37], [-6, -6], [-21, 9], [14, -12], [-11, 8], [23, -3], [3, 11], [10, -30], [-7, 25], [23, -16], [1, -15], [-10, 14], [-6, -6], [-10, 26], [7, 19], [-24, 30], [-32, 35], [11, -26], [-3, 0], [-38, 15], [15, 0], [-39, -1], [-9, -14], [26, 4], [13, -13], [-10, 12], [6, 14], [-40, 16], [47, 2], [-27, 12], [36, -13], [42, -6], [29, -20], [11, -8], [7, -2], [34, -25], [-16, -4], [9, 21], [1, 29], [-21, 26], [15, 24], [15, -3], [16, -15], [-32, 12], [-34, -4], [-21, -18], [46, -12], [-45, 17], [-35, 14], [24, -20], [9, -20], [3, -38], [6, 2], [23, 9], [34, -36], [42, -24], [4, 28], [35, -36], [-2, -38], [25, -26], [27, 5], [18, 6], [-28, 30], [17, 16], [-7, 41], [-17, 20], [-21, 8], [36, 12], [12, 17], [-36, 48], [-27, -21], [15, 15], [38, 1], [-42, 9], [39, -47], [12, -40], [-27, 20], [-21, -9], [45, 13], [-10, -10], [-48, 21], [3, -18], [29, -5], [-23, 43], [28, -30], [-15, 25], [-35, 10], [-7, 1], [-18, 0], [15, 24], [-55, 21], [33, -24], [-66, 3], [-37, -1], [-51, 32], [55, 0], [-11, -16], [56, 4], [8, -41], [-16, -26], [22, 9], [39, 11], [10, 22], [-24, 36], [9, 9], [-30, -13], [-45, 18], [49, 15], [49, 2], [26, 12], [-5, 2], [-3, 22], [40, 5], [-17, -8], [50, 1], [-35, 32], [36, -52], [-65, 0], [-16, -38], [49, -23], [16, -23], [5, -12], [-31, -8], [4, 9], [-31, 56], [5, 3], [-41, 7], [64, -11], [27, 18], [-5, 46], [39, 4], [-21, 22], [48, -6], [-35, -22], [65, 9], [39, -42], [9, -18], [10, 14], [34, 18], [29, -38], [-48, 20], [-37, -13], [28, -22], [15, -56], [-45, 18], [30, -4], [-18, 51], [-35, 65], [27, -36], [-45, 18], [31, -19], [-40, -11], [13, -40], [29, -68], [-45, 42], [-9, 14], [22, 6], [48, -22], [14, -44], [33, 14], [-58, 41], [-9, 6], [-33, -13], [-9, -2], [50, -4], [-27, 60], [10, 38], [1, 25], [-41, 22], [-39, 40], [11, -4], [-24, -1], [37, -58], [2, 29], [-19, 18], [42, -9], [29, 20], [51, -14], [-30, -20], [-12, 4], [47, -36], [-57, 42], [-6, -21], [-32, -1], [-42, 54], [33, 6], [-11, -7], [63, -30], [-60, -5], [-55, 29], [-1, -48], [-22, 25], [-25, 56], [24, -18], [-55, -16], [42, -12], [10, -46], [6, 17], [-2, 24], [17, 2], [30, 4], [-25, -18], [22, 30], [28, -56], [16, -29], [-27, -9], [-16, 14], [-40, 36], [11, 10], [21, 39], [26, 13], [10, 39], [10, 16], [-14, -33], [33, 30], [-2, 38], [41, 14], [22, -62], [24, -4], [-18, 30], [67, 4], [41, 16], [4, -4], [69, 6], [1, 10], [-15, 63], [-6, 21], [-84, 3], [-8, -25], [-16, 9], [24, -16], [-58, 4], [-32, -11], [1, -19], [9, -60], [-63, 50], [-15, 53], [52, 4], [-31, -20], [38, -69], [12, -21], [48, -12], [-36, -2], [-22, 48], [-78, 21], [27, -48], [45, -60], [-29, -5], [-48, -2], [-38, -27], [25, 7], [-5, 25], [66, -29], [30, 0], [-21, 18], [-48, 66], [30, -45], [15, -33], [-7, -20], [7, 50], [7, 52], [63, -34], [-22, 3], [-15, -47], [53, -40], [-5, 14], [1, 43], [21, -45], [-6, -22], [-54, 15], [24, -18], [-40, -9], [-18, 8], [-13, 63], [-57, -12], [83, 0], [-18, 24], [-6, 62], [20, -47], [-29, -19], [-55, 6], [10, 26], [26, 34], [21, 40], [13, 2], [41, -45], [60, 9], [39, -41], [-45, 38], [1, 2], [32, -56], [68, -52], [-12, -33], [33, 22], [-52, 47], [9, 15], [-59, 18], [-53, -19], [-8, 22], [-24, 20], [32, -40], [-63, 3], [18, 51], [-60, 3], [-33, -4]]
